/**
 * The <TightFormInput> component renders an input in ReactTightForm wrappers.
 *
 * Supports label, help, text input add-ons, validation and use as wrapper.
 *
 * If you don't want the <TightFormInput> to act as a wrapper, set the prop
 * named "standalone" and this component will take care of setting up the
 * appropriate CSS classes
 *
 * Use "getValue()" or "getChecked()" to get the current state. The helper
 * method "getInputDOMNode()" returns the internal input element.
 *
 * **NOTE** This component does not support file, multi-select and textarea.
 */

import React from 'react';
import classNames from 'classnames';

import TightFormGroup from './TightFormGroup';
import TightFormLabel from './TightFormLabel';

function defaultWrapperTemplate(props, input) {
  return input;
}

function isCheckboxOrRadio(props) {
  return props.type === 'checkbox' || props.type === 'radio';
}

function makeInputClassName(props) {
  const validationClassName = props.validationState
    ? 'has-' + props.validationState
    : undefined;
  return classNames(props.className, {
    "TightForm-control": !isCheckboxOrRadio(props),
    "TightForm-border-right-none": props.noRightBorder // does nothing if checkbox/radio
  }, validationClassName);
}

const TightFormInput = React.createClass({

  propTypes: {
    /**
     * This is used primarily for <select> fields (eg. <option> list). If this
     * component is not of type "select", this will be ignored.
     */
    children: React.PropTypes.node,

    /**
     * The css class name of the <input>/<select> element.
     */
    className: React.PropTypes.string,

    /**
     * Disables the input field if set to `true`.
     *
     * defaults to `false`
     */
    disabled: React.PropTypes.bool,

    /**
     * Props to pass into the <TightFormGroup> component.
     */
    groupProps: React.PropTypes.object,

    /**
     * The id prop for the input field. This is *REQUIRED* if this input type
     * is "checkbox" or "radio"
     */
    id: props => {
      if (isCheckboxOrRadio(props)) {
        if (!props.id) {
          return new Error(`checkbox or radio inputs require the "id" prop to
            be set.`);
        }
      }
    },

    /**
     * Label for this input.
     *
     * If this is not set, no <TightFormLabel> element will be rendered.
     */
    label: React.PropTypes.node,

    /**
     * Props for the <TightFormLabel> component. This is only valid if the
     * "label" prop has been set.
     */
    labelProps: React.PropTypes.object,

    /**
     * Set this to `true` to set CSS "border-right" prop to "0 none"
     *
     * defaults to `false`
     */
    noRightBorder: React.PropTypes.bool,

    /**
     * Optional placeholder text to display to the user. This applies to
     * input fields that support this prop (eg. "text", "number", etc.).
     */
    placeholder: React.PropTypes.string,

    /**
     * Provide a name for the input element.
     *
     * If this component is a "radio" type, this field *should* be set.
     */
    name: React.PropTypes.string,

    /**
     * set this to `true` to prevent this component from acting as a
     * <TightFormGroup> wrapper.
     *
     * defaults to `false`
     */
    standalone: React.PropTypes.bool,

    /**
     * Override the inline-styles of the root element.
     */
    style: React.PropTypes.object,

    /**
     * Specifies the type of input to display such as "password" or "text".
     */
    type: React.PropTypes.oneOf([
      'checkbox',
      'email',
      'number',
      'password',
      'radio',
      'select',
      'text'
    ]),

    /**
     * This component includes validation styles for error, warning, and
     * success states on form controls and labels.
     *
     * If this is a "standalone" component, we will apply the state to the
     * element.
     */
    validationState: React.PropTypes.oneOf([
      'success',
      'warning',
      'error'
    ]),

    /**
     * Sometimes you might need to render your inputs within a container /
     * wrapper.
     *
     * For example, suppose you want an <input> "typeahead" feature or display
     * a pretty "search" font-awesome icon. You can do so by passing in a
     * custom "template" function which will receive the same props that
     * <TightFormInput> has received.
     *
     * **NOTE** DO NOT MUTATE "props".
     *
     * @param {object} props
     * @param {ReactElement} input
     */
    wrapperTemplate: React.PropTypes.func
  },

  getDefaultProps() {
    return {
      disabled: false,
      noRightBorder: false,
      standalone: false,
      wrapperTemplate: defaultWrapperTemplate
    };
  },

  render() {
    const {
      groupProps={},
      standalone,
      validationState,
      wrapperTemplate
    } = this.props;
    let input = wrapperTemplate(this.props, this.renderInput());

    if (standalone) {
      return input;
    }

    /**
     * Checkbox and radio input come back as an Array.
     */
    if (React.isValidElement(input)) {
      input = React.cloneElement(input, {key: 'input'});
    }

    return (
      <TightFormGroup {...groupProps} validationState={validationState}>
        {this.isCheckboxOrRadio() ?
          this.renderLabel(input) :
          [this.renderLabel(), input]
        }
      </TightFormGroup>
    );
  },

  renderLabel(children) {
    const {
      id,
      noRightBorder,
      label,
      labelProps={}
    } = this.props;

    if (this.isCheckboxOrRadio()) {
      const classes = classNames(labelProps.className, {
        "TightForm-control-checkbox-radio": true,
        "TightForm-border-right-none": noRightBorder
      });

      return (
        <div {...labelProps} key="label" className={classes}>
          {children}
          {label &&
            <label htmlFor={id} style={{cursor: 'pointer'}}>
              {label}
            </label>
          }
        </div>
      );
    } else {
      return label ? (
        <TightFormLabel {...labelProps} key="label" htmlFor={id}>
          {label}
        </TightFormLabel>
      ) : children;
    }
  },

  renderInput() {
    let {
      children,
      className,
      disabled,
      groupProps,
      noRightBorder,
      id,
      label,
      labelProps,
      standalone,
      type,
      validationState,
      wrapperTemplate,
      ...others
    } = this.props;

    switch (type) {
      case 'select':
        return (
          <select
            {...others}
            ref="input"
            id={id}
            disabled={disabled}
            className={makeInputClassName(this.props)}
          >
            {children}
          </select>
        );
      case 'checkbox':
        return [
          <input
            {...others}
            key="checkbox-input"
            ref="input"
            id={id}
            disabled={disabled}
            type={type}
            className={makeInputClassName(this.props)}
          />,
          <label key="checkbox-label" htmlFor={id} />
        ];
      case 'radio':
        return [
          <input
            {...others}
            key="radio-input"
            ref="input"
            id={id}
            disabled={disabled}
            type={type}
            className={makeInputClassName(this.props)}
          />,
          <label key="radio-label" htmlFor={id} />
        ];
      // case 'email':
      // case 'number':
      // case 'password':
      // case 'text':
      default:
        return (
          <input
            {...others}
            ref="input"
            id={id}
            disabled={disabled}
            type={type}
            className={makeInputClassName(this.props)}
          />
        );
    }
  },

  /**
   *
   */
  getInputDOMNode() {
    return this.refs.input;
  },

  /**
   * TODO: immplement multi-select.
   */
  getValue() {
    const { type } = this.props;

    if (type) {
      return this.getInputDOMNode().value;
    }
    throw new Error('Cannot use getValue without specifying input type.');
  },

  /**
   *
   * @return {*}
   */
  isChecked() {
    return this.getInputDOMNode().checked;
  },

  /**
   *
   */
  isCheckboxOrRadio() {
    return isCheckboxOrRadio(this.props);
  }

});

export default TightFormInput;